import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.net.URL;
import java.util.Calendar;
import java.util.GregorianCalendar;

public class DownloadTRMM {

	/**
	 * @param args
	 * @throws IOException
	 */
	public static void main(String[] args) {
		if (args.length == 2) {
			String date = args[0];
			if (date.length() != 8) {
				System.out.println("'" + date + "' is not in the correct 8-digit-datestamp-format. Example: 20120101");
				return;
			}
			String destination = args[1];
			//String url = "http://disc2.nascom.nasa.gov/opendap/TRMM_L3/TRMM_3B42/";
			String url = "ftp://disc2.nascom.nasa.gov/ftp/data/s4pa/TRMM_L3/TRMM_3B42/";
			String year = date.substring(0, 4);
			String month = date.substring(4, 6);
			String day = date.substring(6, 8);
			Calendar cal = new GregorianCalendar(Integer.parseInt(year), Integer.parseInt(month) - 1, Integer.parseInt(day));
			String julianday = String.format("%03d", cal.get(Calendar.DAY_OF_YEAR));
			cal.add(Calendar.DAY_OF_YEAR, 1);
			String nextDate = String.format("%04d%02d%02d", cal.get(Calendar.YEAR), cal.get(Calendar.MONTH) + 1, cal.get(Calendar.DAY_OF_MONTH));
			url += year + "/" + julianday + "/";
			String inputPostfix = ".7.HDF";
			String outputPostfix = inputPostfix;
			String[] times = { "03", "06", "09", "12", "15", "18", "21" };
			for (String time : times) {
				if (urlExists(url + "3B42." + date + "." + time + inputPostfix))
					break;
				else
					inputPostfix = ".7A.HDF";
				if (urlExists(url + "3B42." + date + "." + time + inputPostfix))
					break;
				else
					inputPostfix = ".7.HDF";
			}
			for (String time : times) {
				String inputUrl = url + "3B42." + date + "." + time + inputPostfix;
				String outputFile = "3B42." + date + "." + time + outputPostfix;
				DownloadFile(inputUrl, outputFile, destination, false);
			}
			String inputUrl = url + "3B42." + nextDate + "." + "00" + inputPostfix;
			String outputFile = "3B42." + date + "." + "24" + outputPostfix;
			DownloadFile(inputUrl, outputFile, destination, false);
		} else {
			System.out.println("DownloadTRMM downloads all 8 HDF files on a date from the TRMM  website.");
			System.out.println("The output is in HDF format.");
			System.out.println("The last file of a date is renamed.");
			System.out.println("Originally its filename is timestamped as 00 hrs of the next day,");
			System.out.println("this is changed to be the 24th hour of the current day.");
			System.out.println("Usage   : java -jar DownloadTRMM.jar <datestamp> <destination_path>");
			System.out.println("Example : java -jar DownloadTRMM.jar 20120101 C:\\Temp");
			System.out.println("Note: use quotes for a destination_path with spaces.");
		}
	}

	private static boolean urlExists(String file) {
		boolean exists = true;
		try {
			URL fileUrl = new URL(file);
			fileUrl.openStream();
		} catch (IOException e) {
			exists = false;
		}
		return exists;
	}

	private static final int bufSize = 65536;
	private static final byte[] buf = new byte[bufSize];

	private static void DownloadFile(String file, String outFile, String destination, boolean force) {
		String slash = (destination != null && (destination.endsWith("/") || destination.endsWith("\\"))) ? "" : "\\";
		outFile = destination + slash + outFile;
		File f = new File(outFile);
		if (force || (!f.exists())) {
			System.out.println("Downloading file '" + file.substring(file.lastIndexOf('/') + 1) + "' as '" + outFile.substring(outFile.lastIndexOf('\\') + 1) + "'");
			try {
				URL fileUrl = new URL(file);
				BufferedInputStream bis = new BufferedInputStream(fileUrl.openStream());
				BufferedOutputStream ost = new BufferedOutputStream(new FileOutputStream(outFile));

				int total = bis.read(buf);
				while (total > -1) {
					ost.write(buf, 0, total);
					total = bis.read(buf);
				}
				ost.close();
				bis.close();
			} catch (IOException e) {
				System.out.println(e.toString());
			}
		} else {
			System.out.println("Skipping file '" + outFile.substring(outFile.lastIndexOf('\\') + 1) + "' as it already exists");
		}
	}
}
